/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 04/03/23.
//

#ifndef FGGL_GUI_MODEL_PARSER_HPP
#define FGGL_GUI_MODEL_PARSER_HPP

#include <fggl/gui/model/structure.hpp>

namespace fggl::gui::model {

	class WidgetFactory {
		public:
			inline Widget* build(std::string templateName) {
				return new Widget(m_templates.at(templateName));
			}

			inline Widget* buildEmpty() {
				return new Widget();
			}

			inline void push(std::string name, const Widget& definition) {
				m_templates[name] = definition;
			}

		private:
			std::map<std::string, Widget> m_templates;
	};

	Widget* parseFile(WidgetFactory& factory, const std::string& path);

} // namespace fggl::gui::model

#endif //FGGL_GUI_MODEL_PARSER_HPP
