/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 04/03/23.
//

#ifndef FGGL_GUI_RENDERER_RENDERER_HPP
#define FGGL_GUI_RENDERER_RENDERER_HPP

#include <fggl/gui/model/structure.hpp>
#include <fggl/gfx/paint.hpp>

namespace fggl::gui::renderer {

	struct Box {
		float top;
		float right;
		float bottom;
		float left;

		inline Box trim(Box border) const {
			return {
				top + border.top,
				right - border.right,
				bottom - border.bottom,
				left + border.left
			};
		}

		inline Box expand(Box border) const {
			return {
				top - border.top,
				right + border.right,
				bottom + border.bottom,
				left - border.left
			};
		}

		inline float width() const {
			return right - left;
		}

		inline float height() const {
			return bottom - top;
		}
	};

	inline math::vec2 get_vec2(const model::Widget& root, const std::string& name) {
		if ( !root.hasAttr(name) ) {
			return {};
		}
		return root.get<math::vec2>(name);
	}

	inline math::vec3 get_vec3_rgb(const model::Widget& root, const std::string& name) {
		if ( !root.hasAttr(name) ) {
			return {};
		}
		return root.get<math::vec3>(name);
	}

	inline Box get_box(const model::Widget& root, const std::string& name) {
		return {
			root.get_or_default<float>(name + "::top"),
			root.get_or_default<float>(name + "::right"),
			root.get_or_default<float>(name + "::bottom"),
			root.get_or_default<float>(name + "::left")
		};
	}

	inline Box getBounds(math::vec2 pos, math::vec2 size) {
		return {
			pos.y,
			pos.x + size.x,
			pos.y + size.y,
			pos.x
		};
	}

	void draw_box(gfx::Path2D &path, glm::vec2 topLeft, glm::vec2 bottomRight);

	void draw_border_patch(gfx::Paint& paint, Box& bounds, Box& size, math::vec3 colour);

	void draw_border_solid(gfx::Paint& paint, Box& bounds, Box& size, math::vec3 colour);

	void draw_background_solid(gfx::Paint& paint, Box& bounds, math::vec3 colour);

	void visit(const model::Widget& root, gfx::Paint& paint, Box offset = {0.0F, 1024.0F, 768.0F, 0.0F});

} // namespace fggl::gui::renderer

#endif //FGGL_GUI_RENDERER_RENDERER_HPP
